@@ -34,53 +34,60 @@
 #include "Common.h"
 #include "FilterSelector/FiltersModel.h"
 #include "Globals.h"
+#include "Logger.h"
+#include "Utils.h"
+#include "Widgets/LanguageSelectionWidget.h"
 #include "gmic_qt.h"
 #include "gmic.h"
 
-FiltersModelReader::FiltersModelReader(FiltersModel & model) : _model(model)
-{
-}
+FiltersModelReader::FiltersModelReader(FiltersModel & model) : _model(model) {}
 
 void FiltersModelReader::parseFiltersDefinitions(QByteArray & stdlibArray)
 {
+  TIMING;
   QBuffer stdlib(&stdlibArray);
-  stdlib.open(QBuffer::ReadOnly);
+  stdlib.open(QBuffer::ReadOnly | QBuffer::Text);
   QList<QString> filterPath;
 
-  QString language;
-  QList<QString> languages = QLocale().uiLanguages();
-  if (languages.size()) {
-    language = languages.front().split("-").front();
-  } else {
+  QString language = LanguageSelectionWidget::configuredTranslator();
+  if (language.isEmpty()) {
     language = "void";
   }
-  if (!stdlibArray.contains(QString("#@gui_%1").arg(language).toLocal8Bit())) {
-    language = "en";
-  }
 
   // Use _en locale if not localization for the language is found.
+  if (!stdlibArray.startsWith(QString("#@gui_%1").arg(language).toLocal8Bit()) && !stdlibArray.contains(QString("\n#@gui_%1").arg(language).toLocal8Bit())) {
+    language = "en";
+  }
 
-  QString buffer = stdlib.readLine(4096);
+  QString buffer = readBufferLine(stdlib);
   QString line;
 
-  QRegExp folderRegexpNoLanguage("^..gui[ ][^:]+$");
-  QRegExp folderRegexpLanguage(QString("^..gui_%1[ ][^:]+$").arg(language));
+  QRegExp folderRegexpNoLanguage("^#@gui[ ][^:]+$");
+  QRegExp folderRegexpLanguage(QString("^#@gui_%1[ ][^:]+$").arg(language));
 
-  QRegExp filterRegexpNoLanguage("^..gui[ ][^:]+[ ]*:.*");
-  QRegExp filterRegexpLanguage(QString("^..gui_%1[ ][^:]+[ ]*:.*").arg(language));
+  QRegExp filterRegexpNoLanguage("^#@gui[ ][^:]+[ ]*:.*");
+  QRegExp filterRegexpLanguage(QString("^#@gui_%1[ ][^:]+[ ]*:.*").arg(language));
+
+  QRegExp hideCommandRegExp(QString("^#@gui_%1[ ]+hide\\((.*)\\)").arg(language));
+
+  QVector<QString> hiddenPaths;
 
   const QChar WarningPrefix('!');
   do {
     line = buffer.trimmed();
     if (line.startsWith("#@gui")) {
-      if (folderRegexpNoLanguage.exactMatch(line) || folderRegexpLanguage.exactMatch(line)) {
+      if (hideCommandRegExp.exactMatch(line)) {
+        QString path = hideCommandRegExp.cap(1);
+        hiddenPaths.push_back(path);
+        buffer = readBufferLine(stdlib);
+      } else if (folderRegexpNoLanguage.exactMatch(line) || folderRegexpLanguage.exactMatch(line)) {
         //
         // A folder
         //
         QString folderName = line;
-        folderName.replace(QRegExp("^..gui[_a-zA-Z]{0,3}[ ]"), "");
+        folderName.replace(QRegExp("^#@gui[_a-zA-Z]{0,3}[ ]"), "");
 
-        while (folderName.startsWith("_") && filterPath.size()) {
+        while (folderName.startsWith("_") && !filterPath.isEmpty()) {
           folderName.remove(0, 1);
           filterPath.pop_back();
         }
@@ -90,14 +97,14 @@ void FiltersModelReader::parseFiltersDefinitions(QByteArray & stdlibArray)
         if (!folderName.isEmpty()) {
           filterPath.push_back(folderName);
         }
-        buffer = stdlib.readLine(4096);
+        buffer = readBufferLine(stdlib);
       } else if (filterRegexpNoLanguage.exactMatch(line) || filterRegexpLanguage.exactMatch(line)) {
         //
         // A filter
         //
         QString filterName = line;
         filterName.replace(QRegExp("[ ]*:.*$"), "");
-        filterName.replace(QRegExp("^..gui[_a-zA-Z]{0,3}[ ]"), "");
+        filterName.replace(QRegExp("^#@gui[_a-zA-Z]{0,3}[ ]"), "");
 
         const bool warning = filterName.startsWith(WarningPrefix);
         if (warning) {
@@ -105,12 +112,12 @@ void FiltersModelReader::parseFiltersDefinitions(QByteArray & stdlibArray)
         }
 
         QString filterCommands = line;
-        filterCommands.replace(QRegExp("^..gui[_a-zA-Z]{0,3}[ ][^:]+[ ]*:[ ]*"), "");
+        filterCommands.replace(QRegExp("^#@gui[_a-zA-Z]{0,3}[ ][^:]+[ ]*:[ ]*"), "");
 
         QList<QString> commands = filterCommands.split(",");
 
         QString filterCommand = commands[0].trimmed();
-        if (commands.size() == 0) {
+        if (commands.isEmpty()) {
           commands.push_back("_none_");
         }
         if (commands.size() == 1) {
@@ -143,10 +150,10 @@ void FiltersModelReader::parseFiltersDefinitions(QByteArray & stdlibArray)
         // Read parameters
         QString parameters;
         do {
-          buffer = stdlib.readLine(4096);
+          buffer = readBufferLine(stdlib);
           if (buffer.startsWith(start)) {
             QString parameterLine = buffer;
-            parameterLine.replace(QRegExp("^..gui[_a-zA-Z]{0,3}[ ]*:[ ]*"), "");
+            parameterLine.replace(QRegExp("^#@gui[_a-zA-Z]{0,3}[ ]*:[ ]*"), "");
             parameters += parameterLine;
           }
         } while ((buffer.startsWith(start) || buffer.startsWith("#") || (buffer.trimmed().isEmpty() && !stdlib.atEnd())) && !folderRegexpNoLanguage.exactMatch(buffer) &&
@@ -163,12 +170,56 @@ void FiltersModelReader::parseFiltersDefinitions(QByteArray & stdlibArray)
         filter.setWarningFlag(warning);
         filter.build();
         _model.addFilter(filter);
-
       } else {
-        buffer = stdlib.readLine(4096);
+        buffer = readBufferLine(stdlib);
       }
     } else {
-      buffer = stdlib.readLine(4096);
+      buffer = readBufferLine(stdlib);
     }
   } while (!buffer.isEmpty());
+
+  // Remove hidden filters from the model
+  for (const QString & path : hiddenPaths) {
+    const size_t count = _model.filterCount();
+    QList<QString> pathList = path.split("/", QString::SkipEmptyParts);
+    _model.removePath(pathList);
+    if (_model.filterCount() == count) {
+      Logger::log(QString("[%1]./warning/ While hidding filter, name or path not found: \"%2\"\n").arg(GmicQt::pluginCodeName()).arg(path));
+    }
+  }
+  TIMING;
+}
+
+QString FiltersModelReader::readBufferLine(QBuffer & buffer)
+{
+  // QBuffer::readline(max_size) may be very slow, in debug mode, when max_size
+  // is too big (e.g. 1MB). We read large lines in multiple calls.
+  QString result;
+  QString text;
+  do {
+    text = buffer.readLine(1024);
+    result.append(text);
+  } while (!text.isEmpty() && !text.endsWith("\n"));
+
+  // Merge comment lines ending with '\'
+  while (result.startsWith("#") && result.endsWith("\\\n")) {
+    char nextLinePrefix;
+    buffer.peek(&nextLinePrefix, 1);
+    if (nextLinePrefix != '#') {
+      return result;
+    }
+    result.chop(2);
+    QString nextLine;
+    do {
+      text = buffer.readLine(1024);
+      nextLine.append(text);
+    } while (!text.isEmpty() && !text.endsWith("\n"));
+    int ignoreCount = 1;
+    const int limit = nextLine.length() - nextLine.endsWith("\n");
+    while (ignoreCount < limit && nextLine[ignoreCount] <= ' ') {
+      ++ignoreCount;
+    }
+    result.append(nextLine.rightRef(nextLine.length() - ignoreCount));
+  }
+  return result;
 }
